// 
//  ERRE.h
//  
//  ERRE - ERror REporting engine for C++
//  ERRE is a error reporting "engine" written in C++. With this engine you can easily report errors ;)
//
//  Author:
//	 Daniel Bugl  <Daniel.Bugl@gmail.com>
//
//  Copyright (C) 2011 Daniel Bugl <Daniel.Bugl@gmail.com>
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#define E_MESSAGE 0
#define E_NOTE 1
#define E_INFO 2
#define E_SUCCESS 3
#define E_WARNING 4
#define E_ERROR 5
#define E_NORMAL 6
#define E_NORMALLINE 7

bool ERRE_error_reporting[9];

const char * ERRE_projectname = "ERRE";

bool ERRE_reporting_stderr = false;

void ERRE_SetProjectName(const char * name)
{
    ERRE_projectname = name;
}

bool ERRE_ReportingIsAllowed(int type)
{
    if (ERRE_error_reporting[type] == true) return true;
    else return false;
}

void ERRE_ReportingSetAllowed(int type, bool set)
{
    ERRE_error_reporting[type] = set;
}

void ERRE_ReportingAllowAll()
{
    for (int i = 0; i < 9; i++) ERRE_error_reporting[i] = true;
}

void ERRE_ReportingDisallowAll()
{
    for (int i = 0; i < 9; i++) ERRE_error_reporting[i] = false;
}

void ERRE_ReportingAllow(int type)
{
    ERRE_ReportingSetAllowed(type, true);
}

void ERRE_ReportingDisallow(int type)
{
    ERRE_ReportingSetAllowed(type, false);
}

void ERRE_ReportingAllowStderr()
{
    ERRE_reporting_stderr = true;
}

void ERRE_ReportingDisallowStderr()
{
    ERRE_reporting_stderr = false;
}

void ERRE_printf(int type, const char *s, ...)
{
    char buffer[256];

    va_list pointer;
    va_start(pointer, s);
    vsprintf(buffer, s, pointer);
    va_end(pointer);
    if (ERRE_ReportingIsAllowed(type))
    {
	switch(type)
	{
	case E_MESSAGE: printf("[%s::Message] %s\n", ERRE_projectname, buffer); break;
	case E_NOTE: printf("[%s::Note] %s\n", ERRE_projectname, buffer); break;
	case E_INFO: printf("[%s::Info] %s\n", ERRE_projectname, buffer); break;
	case E_SUCCESS: printf("[%s::Success] %s\n", ERRE_projectname, buffer); break;
	case E_WARNING: printf("[%s::Warning] %s\n", ERRE_projectname, buffer); break;
	case E_ERROR:
		if (ERRE_reporting_stderr == true) fprintf(stderr, "[%s::Error] %s\n", ERRE_projectname, buffer);
		printf("[%s::Error] %s\n", ERRE_projectname, buffer);
	break;
	case E_NORMAL: printf("%s", buffer); break;
	case E_NORMALLINE: printf("%s\n", buffer); break;
	}
    }
}

void eprintf(int type, const char *s, ...)
{
    char buffer[256];

    va_list pointer;
    va_start(pointer, s);
    vsprintf(buffer, s, pointer);
    va_end(pointer);

    char *str;

    strcpy(str, buffer);

    ERRE_printf(type, str);
}
