//
//  openmary.cpp
//
//  Author:
//       Jacky Alcine <jackyalcine@gmail.com>
//	 Daniel Bugl  <Daniel.Bugl@gmail.com>
//
//  Copyright (c) 2011
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  --> View more about this project at https://wiki.ubuntu.com/SpeechControl <--
//
	/* ERRE - ERror REporting for C++ by Daniel Bugl (Daniel0108) */
//#include "ERRE.h"
#include "openmary.h"
#include <stdexcept>

using namespace std;

using std::string;
using std::cout;

namespace openMary {
	static string theBigData;

	// Code below provided by Daniel Bugl. Thanks :)
	size_t handle_data(void *ptr, size_t size, size_t nmemb, void *userData){
   		int numbytes = size * nmemb;
   		char *theStr = (char *) ptr;
   		char lastchar = *( (char *) ptr + numbytes - 1);

  		*( (char *) ptr + numbytes - 1) = '\0';
		*( (char *) ptr + numbytes - 1) = lastchar;

		userData = ptr;
		openMary::theBigData = (char *) userData;
		*((char *) userData) = *(const_cast<char *>(openMary::theBigData.c_str()));
	
		return numbytes;
	}

	Engine::Engine() {
		ERRE_ReportingAllowAll();
		ERRE_ReportingAllowStderr();
		this->_maryuri = "http://mary.dfki.de:59125/";
		this->curlHandle = curl_easy_init();
		if (this->curlHandle == NULL){
			eprintf(E_ERROR, "A openMary::Engine could not be created because the cURL library failed to initialize.");
		}

		eprintf(E_INFO, "Engine created with cURL.");

		curl_easy_setopt(this->curlHandle,CURLOPT_WRITEFUNCTION,&openMary::handle_data);
		curl_easy_setopt(this->curlHandle,CURLOPT_VERBOSE,1);
		test();
		jumpStart();
	}

	Engine::~Engine(){
		curl_easy_cleanup(this->curlHandle);
	}

	void Engine::jumpStart(){

	}

	void Engine::start() {
		eprintf(E_ERROR, "This feature is not implemented yet.");
		theBigData.clear();
	}

	const bool Engine::isActive(){
		return true;
	}

	const string Engine::getHttpString(string uri, string query){
   		string returnValue = "";

   		if (query.length() > 0){
   			uri.append("?");
   			uri.append(query);
   		}
   		const char *theURL = uri.c_str();


   		eprintf(E_INFO,"Opening '%s'...",theURL);
   		curl_easy_setopt(this->curlHandle,CURLOPT_URL, theURL);
   		curl_easy_setopt(this->curlHandle,CURLOPT_WRITEDATA,&returnValue);
   		eprintf(E_INFO,"Performing '%s' ...",theURL);
		CURLcode res = curl_easy_perform(this->curlHandle);
		eprintf(E_INFO,"Peformed.");
	    	cout << "Returned " << returnValue << ", " << res << endl;

	    	if (res != 0){
				eprintf(E_ERROR, "cURL-Error %d", res);
				returnValue = "";
	    	}

	    	Engine::theData.empty();
	    	return returnValue;
	}

	void Engine::getHttpFile(string uri, string filename){
		eprintf(E_INFO,"Opening remote file '%s'...",uri.c_str());
		uri = this->_maryuri + uri;

		string data = this->getHttpString(uri);

		FILE* fileHandle = fopen((char*)filename.c_str(),"w");
			fprintf(fileHandle, "%s", data.c_str());
		fclose(fileHandle);
	}

	void Engine::setMaryUri(const string uri){
		this->_maryuri = uri;
		curl_easy_setopt(this->curlHandle,CURLOPT_URL,uri.data());
	}

	const string Engine::getMaryURI(){
		return this->_maryuri;
	}

	const string Engine::getVersion(){
		return this->doRequest(OPENMARY_VERSION);
	}

	void Engine::test(){
		eprintf(E_MESSAGE,"Starting test...");

		try {
			this->getHttpFile("exampletext?datatype=AUDIO_WAV&locale=en_US","audio.wav");
		}
		catch (exception &exception){
			eprintf(E_ERROR,"An serious exception occurred when attempting to test the engine's capibilities.\nError: %s",exception.what());
		}

		eprintf(E_MESSAGE,"Test ended.");
	}

	const string Engine::doRequest(OPENMARY_REQUEST theRequest, ...){
	    va_list attributes;
	    int recievedCount = 0;
	    string returnValue;
	    string url = this->_maryuri;
	    string query;
	string datatype("RAWMARYXML");
    	string locale("en-us");
    	string voice("dfki-bundesliga");

    	va_start(attributes,theRequest);
    	eprintf(E_MESSAGE,"Executing request %d..",theRequest);

	    switch (theRequest){
		    case OPENMARY_EXAMPLETEXT:
		    	if (recievedCount >= 1)	datatype = (va_arg(attributes,char*));
		    	if (recievedCount >= 2)	locale = (va_arg(attributes,char*));
		    	if (recievedCount == 3)	voice = (va_arg(attributes,char*));

		    	url.append("exampletext");
		    	query.append("datatype=" + datatype);
		    	query.append("&locale=" + locale);
		    	query.append("&voice=" + voice);
		    	break;

		    case OPENMARY_FEATURES:
		    	if (recievedCount >= 1)	locale = (va_arg(attributes,char*));
		    	if (recievedCount == 2)	voice = (va_arg(attributes,char*));

				url.append("features");
		    	query.append("locale=" + locale);
		    	query.append("&voice=" + voice);
		    	break;

		    default:
		    case OPENMARY_VERSION:
		    	url.append("version");
		    	break;
	    }

	    va_end(attributes);

	    return this->getHttpString(url,query);
	}

}

